/* eslint-disable import/no-named-as-default-member */
import { dirname, join } from 'path';
import { fileURLToPath } from 'url';
import fs from 'fs-extra';

const generateFunctions = async (dest, netlifyConfig) => {
  const __dirname = dirname(fileURLToPath(import.meta.url));
  const src = join(__dirname, 'edge-functions');

  await fs.ensureDir(dest);
  await fs.copy(src, dest);

  netlifyConfig.edge_functions = [
    ...(netlifyConfig.edge_functions || []),
    ...['metadata', 'stories'].map((file) => ({
      path: `/${file}.json`,
      function: 'set-storybook-headers',
    })),
  ];
};

const generateMetadata = async (dest, packageJson) => {
  const { version } = packageJson;
  const url = process.env.DEPLOY_URL;

  if (!version || !url) return null;

  const metadata = { versions: { [`v${version}`]: url } };

  await fs.writeJSON(join(dest, 'metadata.json'), metadata);

  return metadata;
};

export const onBuild = async ({
  constants: { EDGE_FUNCTIONS_SRC = 'netlify/edge-functions', PUBLISH_DIR },
  netlifyConfig,
  packageJson,
}) => {
  // Add metadata.json
  const metadata = await generateMetadata(PUBLISH_DIR, packageJson);

  if (!metadata) return;

  // Add edge functions to set correct headers
  await generateFunctions(EDGE_FUNCTIONS_SRC, netlifyConfig);
};
